/******************************************************************************
    CallWave VoiceMail Widget

    Copyright (C) 2006 CallWave, Inc.  All Rights Reserved.

 ******************************************************************************/

/******************************************************************************
  SMS_LoginView c'tor

    HTML DOM Dependencies:
        WIDGET_loginViewFrame           - <div>
        WIDGET_loginViewStatusTextArea  - <div>
        WIDGET_loginCellNumberTextArea  - <input type="text">
        WIDGET_loginPinTextArea         - <input type="password">

    Call backs:
        onLoginSucceeded( responseMessage )
        onLoginRequired( statusMessage )
        onVerificationRequired( responseMessage )

    Globals:
        VM_GetPreference( key )
        VM_SetPreference( key, value )
        EX_Log( message )
        VM_WIDGET_SOURCE
        VM_WIDGET_VERSION
        VM_WIDGET_OS_VERSION

 ******************************************************************************/
function SMS_LoginView( accountHandler, mobileNumberControlSynchronizer, onLoginSucceeded, onLoginRequired )
{
    try
    {
        this.m_accountHandler       = accountHandler;
        this.m_onLoginSucceeded     = onLoginSucceeded;
        this.m_onLoginRequired      = onLoginRequired;

        this.m_state = this.STATE_MANUAL_LOGIN;

        var self = this;

        this.m_accountHandler.m_onAuthenticateSucceeded   = function( responseMessage )
        {
            self.onAuthenticateSucceeded( responseMessage );
        };

        this.m_accountHandler.m_onAuthenticateFailed      = function( statusMessage  )
        {
            self.onAuthenticateFailed( statusMessage );
        };

        this.m_accountHandler.m_onForgotPinSucceeded      = function( responseMessage )
        {
            self.onForgotPinSucceeded( responseMessage );
        };

        this.m_accountHandler.m_onForgotPinFailed         = function( statusMessage )
        {
            self.onForgotPinFailed( statusMessage );
        };

        function AddEditFieldMethod( self, identifier )
        {
            self['Get' + identifier + 'Obj'] = function()
            {
                return document.getElementById( 'WIDGET_login' + identifier + 'TextArea' );
            };

            self['Get' + identifier + 'Data'] = function()
            {
                return self['Get' + identifier + 'Obj']( ).value;
            };

            self['Set' + identifier + 'Data'] = function( message )
            {
                self['Get' + identifier + 'Obj']( ).value = message;
            };
        }

        AddEditFieldMethod( this, 'Pin' );

        this.m_mobileNumberControl = new SMS_MobileNumberControl( 'WIDGET_loginViewCountryPopup',
                                                                  'WIDGET_loginCellNumberTextArea' );

        mobileNumberControlSynchronizer.AddControl( this.m_mobileNumberControl );
    }
    catch ( e )
    {
        EX_ASSERT_NO_EXCEPTIONS( e, "SMS_LoginView::SMS_LoginView( )" );
    }
}
SMS_LoginView.prototype =
{
    ERROR_MSG_COLOR                   : "#bf0000",
    DEFAULT_MSG_COLOR                 : "#000000",
    DEFAULT_PIN_REQUEST_ERROR_MESSAGE : "Failed to send a PIN reminder. Please try again later.",
    DEFAULT_LOGIN_ERROR_MESSAGE       : "Login failed. Please try again later.",

    SetStatusText : function ( message, color )
    {
        try
        {
            UTILS.SetInnerHTML( 'WIDGET_loginViewStatusTextArea', message, color );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_LoginView::SetStatusText( )" );
        }
    },

    /**************************************************************************
      View repainting methods
     **************************************************************************/
    OnLogout : function( )
    {
        this.m_state = this.STATE_MANUAL_LOGIN;
        this.SetPinData( '' );
    },

    reset:function( )
    {
        try
        {
            $( "#WIDGET_loginViewFrame" ).hide();

            this.SetPinData( '' );

            this.m_accountHandler.StopAuthenticationRequests( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_LoginView::reset( )" );
        }
    },

    refresh:function( )
    {
        try
        {
            $( "#WIDGET_loginViewFrame" ).show();

            this.SetStatusText( "Please enter the following information to login.", "#000000" );

            this.m_mobileNumberControl.GetMobileNumberObj( ).focus( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_LoginView::refresh( )" );
        }
    },

    /**************************************************************************
      Authenticate methods
     **************************************************************************/

    handleAuthenticateSubmit:function( )
    {
        try
        {
            if ( this.m_accountHandler.IsAuthenticationRequestInProgress( ) )
            {
                // ignore
                return;
            }

            this.m_mobileNumberControl.OnFormSubmit( );

            var numberValidationData = this.m_mobileNumberControl.GetValidatedNumber( );

            if ( numberValidationData.ERROR_MESSAGE )
            {
                this.SetStatusText( numberValidationData.ERROR_MESSAGE, this.ERROR_MSG_COLOR );
                return;
            }

            var pin = UTILS_StripNonDigits( this.GetPinData( ) );

            if ( pin.length != this.GetPinData( ).length )
            {
                this.SetPinData( '' );
                this.SetStatusText( "Please enter your 4-10 digit PIN (numeric characters only).", this.ERROR_MSG_COLOR );
                return;
            }

            if ( pin.length < 4 || pin.length > 10 )
            {
                this.SetPinData( '' );
                this.SetStatusText( "Please enter your 4-10 digit PIN.", this.ERROR_MSG_COLOR );
                return;
            }

            VM_SetPreference( PREF_KEY_SENDER_MOBILE, numberValidationData.PHONE_NUMBER_STATE.AsString( ) );

            this.SetStatusText( "Connecting, please wait...", this.DEFAULT_MSG_COLOR );

            this.m_accountHandler.authenticate( numberValidationData.PHONE_NUMBER.GetFullyQualifiedString( ), pin );
        }
        catch ( e )
        {
            this.SetStatusText( "Login failed. Please try again later.", this.ERROR_MSG_COLOR );

            EX_Log( "SMS_LoginView::handleAuthenticateSubmit( ) : " + e.message );
        }
    },

    onAuthenticateSucceeded:function( responseMessage )
    {
        try
        {
            switch ( responseMessage.m_status )
            {
            case MSG_ResponseStatus.SUCCESS:
                {
                    // Save persistent account attributes
                    VM_SetPreference( PREF_KEY_ACCOUNT, responseMessage.m_account );
                    VM_SetPreference( PREF_KEY_U_STRING, responseMessage.m_uString );

                    this.m_onLoginSucceeded( responseMessage );

                    break;
                }

            case MSG_ResponseStatus.GENERAL_ERROR:
                {
                    this.SetStatusText( responseMessage.m_statusMsg, this.ERROR_MSG_COLOR );

                    break;
                }

            case MSG_ResponseStatus.UNEXPECTED_ERROR:
            case MSG_ResponseStatus.RECOVERABLE_ERROR:
                {
                    this.SetStatusText( this.DEFAULT_LOGIN_ERROR_MESSAGE, this.ERROR_MSG_COLOR );

                    break;
                }

            default:
                {
                    this.SetStatusText(
                        ( ( responseMessage.m_statusMsg != null && responseMessage.m_statusMsg.length != 0 ) ?
                            responseMessage.m_statusMsg :
                            this.DEFAULT_LOGIN_ERROR_MESSAGE ) , this.ERROR_MSG_COLOR );

                    break;
                }
            }
        }
        catch ( e )
        {
            var theError = new CWError( 'CWError' );
            theError.Nest( e );

            EX_Log( "SMS_LoginView::onAuthenticateSucceeded( ) : " + theError.message );
        }
    },

    onAuthenticateFailed:function( statusMessage )
    {
        try
        {
            this.SetStatusText( this.DEFAULT_LOGIN_ERROR_MESSAGE, this.ERROR_MSG_COLOR );

            EX_Log( "SMS_LoginView::onAuthenticateFailed( ) : " + statusMessage );
        }
        catch ( e )
        {
            EX_Log( "SMS_LoginView::onAuthenticateFailed( ) : " + e.message );
        }
    },

    /**************************************************************************
      Forgot PIN methods
     **************************************************************************/

    handlePinReminderRequested:function( )
    {
        try
        {
            if ( this.m_accountHandler.IsAuthenticationRequestInProgress( ) )
            {
                // ignore
                return;
            }

            this.m_mobileNumberControl.OnFormSubmit( );

            var numberValidationData = this.m_mobileNumberControl.GetValidatedNumber( );

            if ( numberValidationData.ERROR_MESSAGE )
            {
                this.SetStatusText( numberValidationData.ERROR_MESSAGE, this.ERROR_MSG_COLOR );
                return;
            }

            this.SetPinData( '' );

            VM_SetPreference( PREF_KEY_SENDER_MOBILE, numberValidationData.PHONE_NUMBER_STATE.AsString( ) );

            this.SetStatusText( "Connecting, please wait...", this.DEFAULT_MSG_COLOR );

            this.m_accountHandler.forgotPin( numberValidationData.PHONE_NUMBER.GetFullyQualifiedString( ) );
        }
        catch ( e )
        {
            this.SetStatusText( this.DEFAULT_PIN_REQUEST_ERROR_MESSAGE, this.ERROR_MSG_COLOR );

            EX_Log( "SMS_LoginView::handlePinReminderRequested( ) : Failed to submit manual login : " + e.message );
        }
    },

    onForgotPinSucceeded:function( responseMessage )
    {
        try
        {
            switch ( responseMessage.m_status )
            {
            case MSG_ResponseStatus.SUCCESS:
                {
                    this.SetStatusText( "PIN reminder sent to your cell phone. Enter your PIN when ready.", this.DEFAULT_MSG_COLOR );
                    break;
                }

            case MSG_ResponseStatus.GENERAL_ERROR:
                {
                    this.SetStatusText( responseMessage.m_statusMsg, this.ERROR_MSG_COLOR );

                    break;
                }

            case MSG_ResponseStatus.UNEXPECTED_ERROR:
            case MSG_ResponseStatus.RECOVERABLE_ERROR:
                {
                    this.SetStatusText( this.DEFAULT_PIN_REQUEST_ERROR_MESSAGE, this.ERROR_MSG_COLOR );

                    break;
                }

            default:
                {
                    this.SetStatusText(
                        ( ( responseMessage.m_statusMsg != null && responseMessage.m_statusMsg.length != 0 ) ?
                            responseMessage.m_statusMsg :
                            this.DEFAULT_PIN_REQUEST_ERROR_MESSAGE), this.ERROR_MSG_COLOR );

                    break;
                }
            }
        }
        catch ( e )
        {
            this.SetStatusText( this.DEFAULT_PIN_REQUEST_ERROR_MESSAGE, this.ERROR_MSG_COLOR );

            EX_Log( "SMS_LoginView::onForgotPinSucceeded( ) : " + e );
        }
    },

    onForgotPinFailed:function( statusMessage )
    {
        this.SetStatusText( this.DEFAULT_PIN_REQUEST_ERROR_MESSAGE, this.ERROR_MSG_COLOR );

        EX_Log( "SMS_LoginView::onForgotPinFailed( ) : " + statusMessage );
    }
};